<%@ Page Language="VB" AutoEventWireup="false" ValidateRequest="false" debug="false"%>

<%@ Import Namespace="ReveilleSystem.ExtensionMethods" %>
<%  
    '*********************************************************************
    '*  KTACheckMFPDevices.aspx
    '*
    '*	Tungsten KTA Check MFP Devices via the REST interface. 
    '*
    '*  Sample call:
    '*
    '*	http://localhost/ReveilleTests/Kofax/KTACheckMFPDevices.aspx?Debug=
    '*
    '*  ServerURL=            - The http server url http://<server>
    '*  Path=                 - Path used for REST APIs. Default: TotalAgility
    '*  User=&Password=       - use User=E-<credential> for a credential
    '*  DeviceCountThreshold  - Tests count of devices. <operator>,<threshold> (Default: >=,1)
    '*  VendorCountThreshold  - Tests count of vendors. <operator>,<threshold> (Default: >=,1)
    '*  VersionCountThreshold - Tests count of versions. <operator>,<threshold> (Default: >=,1)
    '*  VersionThreshold      - Tests device count by version. <version>,<operator>,<threshold>;<version2>,<operator2>,<thresold2>;... (no default)
    '*                          <version> can contain wildcards. Use ? for any single character and * for zero or more characters (Ex: 1.* or 2.?0)
    '*  NoContactThreshold    - Tests last contact time based on MinsOld. <operator>,<threshold> (Default: =,0)
    '*                          Valid Operators include: =, <>,>,<,>=,<=,Between
    '*  MinsOld=              - Number of minutes back to test no contact. (Default: 60)
    '*
    '*  MetricDeviceCount=    - Metric for the device count
    '*  MetricVersionCount=   - Metric for the device client version count
    '*  MetricVenderCount=    - Metric for the device client vendor count
    '*  MetricNoContactCount= - Metric for the device no contact count
    '*  Location=             - Optional Location used in the metric and data package name
    '*
    '*	Copyright 2024 Reveille Software
    '*********************************************************************  

    Response.Expires = 0

    Const ENCODE_PREFIX As String = "E-"
    Const DEBUG_MODE As String = "1"

    Dim isDebug As Boolean = (Request("Debug") = DEBUG_MODE)
    Dim serverURL As String = Request("ServerURL")
    Dim apiPath As String = Request("Path")
    Dim userName As String = Request("User")
    Dim password As String = Request("Password")
    Dim monitor As String = Request("Monitor")
    Dim location As String = Request("Location")
    Dim metricDeviceCount As String = Request("MetricDeviceCount")
    Dim metricVendorCount As String = Request("MetricVendorCount")
    Dim metricVersionCount As String = Request("MetricVersionCount")
    Dim metricNoContact As String = Request("MetricNoContactCount")

    Dim deviceThreshold As String = Request("DeviceCountThreshold")
    Dim vendorThreshold As String = Request("VendorCountThreshold")
    Dim versionCountThreshold As String = Request("VersionCountThreshold")
    Dim versionThreshold As String = Request("VersionThreshold")
    Dim noContactThreshold As String = Request("NoContactThreshold")
    Dim minsOld As String = Request("MinsOld")

    Dim serverAddress As String = (serverURL + "/").URLExtractHost

    Dim result As System.Text.StringBuilder = New StringBuilder()
    Dim resultMetric As System.Text.StringBuilder = New StringBuilder()
    Dim resultDebug As System.Text.StringBuilder = New StringBuilder()

    If String.IsNullOrEmpty(deviceThreshold) Then deviceThreshold = ">=,1"
    If String.IsNullOrEmpty(vendorThreshold) Then vendorThreshold = ">=,1"
    If String.IsNullOrEmpty(versionCountThreshold) Then versionCountThreshold = ">=,1"
    If String.IsNullOrEmpty(noContactThreshold) Then noContactThreshold = "=,0"
    If String.IsNullOrEmpty(apiPath) Then apiPath = "TotalAgility"

    Dim theMinsOld As Int32 = 60
    If String.IsNullOrEmpty(minsOld) Then minsOld = "60"
    Int32.TryParse(minsOld, theMinsOld)

    Dim timeout As Integer = 60
    If Not String.IsNullOrEmpty(Request("ReveilleTO")) Then
        Int32.TryParse(Request("ReveilleTO"), timeout)
    End If

    'Set the timeout for this page to run
    If timeout <= 2 Then timeout = 20
    Server.ScriptTimeout = timeout
    Dim restTimeout As Integer = timeout - 2

    If String.IsNullOrEmpty(userName) Then userName = String.Empty
    If Left(userName, 2) = ENCODE_PREFIX Then
        Dim strTemp As String = Mid(userName, 3)
        ReveilleSystem.Credentials.Read(strTemp, userName, password)
    End If

    If isDebug Then
        resultDebug.Append("<br /><br />DEBUG INFORMATION<br />")
        resultDebug.AppendFormat("Parameter ServerURL= '{0}'<br />", Server.HtmlEncode(serverURL))
        resultDebug.AppendFormat("Parameter User= '{0}'<br />", Server.HtmlEncode(userName))
        resultDebug.AppendFormat("Parameter Path= '{0}'<br />", Server.HtmlEncode(apiPath))
        resultDebug.AppendFormat("Parameter Monitor= '{0}'<br />", Server.HtmlEncode(monitor))
        resultDebug.AppendFormat("Parameter DeviceCountThreshold= '{0}'<br />", Server.HtmlEncode(deviceThreshold))
        resultDebug.AppendFormat("Parameter VendorCountThreshold= '{0}'<br />", Server.HtmlEncode(vendorThreshold))
        resultDebug.AppendFormat("Parameter VersionCountThreshold= '{0}'<br />", Server.HtmlEncode(versionCountThreshold))
        resultDebug.AppendFormat("Parameter VersionThreshold= '{0}'<br />", Server.HtmlEncode(versionThreshold))
        resultDebug.AppendFormat("Parameter NoContactThreshold= '{0}'<br />", Server.HtmlEncode(noContactThreshold))
        resultDebug.AppendFormat("Parameter MinsOld= '{0}'<br />", Server.HtmlEncode(minsOld))
        resultDebug.AppendFormat("Parameter Location= '{0}'<br />", Server.HtmlEncode(location))
        resultDebug.AppendFormat("Parameter MetricDeviceCount= '{0}'<br />", Server.HtmlEncode(metricDeviceCount))
        resultDebug.AppendFormat("Parameter MetricVendorCount= '{0}'<br />", Server.HtmlEncode(metricVendorCount))
        resultDebug.AppendFormat("Parameter MetricVersionCount= '{0}'<br />", Server.HtmlEncode(metricVersionCount))
        resultDebug.AppendFormat("Parameter MetricNoContactCount= '{0}'<br />", Server.HtmlEncode(metricNoContact))
        resultDebug.AppendFormat("Script Timeout= '{0}'<br />", timeout)
        resultDebug.AppendFormat("Rest Http Timeout= '{0}'<br />", restTimeout)
    End If

    Dim success As Boolean = True
    Try
        Dim rest As Reveille.KTAREST.REST = New Reveille.KTAREST.REST(serverURL, apiPath, userName, password, monitor, timeout)

        ' Check for Collector Data Package
        '
        Dim errorMsg As String = String.Empty

        If rest.ReadDataPackage(location, "MFPDevices", serverURL, errorMsg) Then

            resultDebug.AppendFormat("Data gathered via Collector at {0}. XmlFile: {1}<br />", rest.TimeCollected, rest.XmlFile)

            result.AppendFormat("<ReveilleCommand>SetRT {0}</ReveilleCommand><br />", rest.ResponseTime)

            If Not String.IsNullOrEmpty(errorMsg) Then
                success = False
                result.AppendFormat("Error: The TotalAgility REST Server {0} failed with error {1}.<br />", Server.HtmlEncode(serverAddress), Server.HtmlEncode(errorMsg).Replace(vbLf, "<br />"))
            End If
        Else
            resultDebug.AppendFormat("Collector XmlFile Not Found: {0}<br />", rest.XmlFile)
            resultDebug.Append("Data gathered via API.<br />")

            Dim StartTime As DateTime = DateTime.Now

            success = rest.GetMFPDevices()

            rest.ResponseTime = (DateTime.Now.Subtract(StartTime).TotalMilliseconds / 1000)

            If Not success Then
                result.AppendFormat("Error: The TotalAgility REST Server {0} failed with error {1}.<br />", Server.HtmlEncode(serverAddress), Server.HtmlEncode(errorMsg).Replace(vbLf, "<br />"))
            End If
        End If

        resultDebug.AppendFormat("- Method: {0}<br />", Server.HtmlEncode(rest.RESTData.Method))
        resultDebug.AppendFormat("- URL: {0}<br />", Server.HtmlEncode(rest.RESTData.Url))
        resultDebug.AppendFormat("- PostData: {0}<br />", Server.HtmlEncode(rest.RESTData.PostData))
        resultDebug.AppendFormat("- StatusCode: {0}<br />", rest.RESTData.StatusCode)
        resultDebug.AppendFormat("- ResponseTime: {0}<br />", rest.ResponseTime)
        resultDebug.AppendFormat("- Response: {0}<br />", Server.HtmlEncode(rest.RESTData.Response).Replace(vbLf, "<br />"))

        If success Then

            Dim data As Reveille.KTAREST.REST.MFPDevices.Root = Newtonsoft.Json.JsonConvert.DeserializeObject(Of Reveille.KTAREST.REST.MFPDevices.Root)(rest.RESTData.Response)

            Dim hostnames As List(Of String) = New List(Of String)()
            Dim vendors As List(Of String) = New List(Of String)()
            Dim versions As List(Of String) = New List(Of String)()
            Dim noContactdeviceNames As List(Of String) = New List(Of String)()
            Dim noContactIPAddresses As List(Of String) = New List(Of String)()
            Dim noContactDeviceAndIp As List(Of String) = New List(Of String)()

            If data Is Nothing Or data.DeviceConfigurations Is Nothing Then
                result.AppendFormat("Error: The TotalAgility REST Server {0} failed with response {1}.<br />", Server.HtmlEncode(serverAddress), Server.HtmlEncode(rest.RESTData.Response).Replace(vbLf, "<br />"))
                Exit Try
            End If

            For Each device In data.DeviceConfigurations
                resultDebug.AppendFormat("Hostname: {0}, Vendor: {1}, Version: {2}<br />",
                                         device.DeviceInfo.DeviceHostname, device.DeviceInfo.DeviceVendor, device.DeviceInfo.ClientVersion)

                If Not hostnames.Contains(device.DeviceInfo.DeviceHostname) Then
                    hostnames.Add(device.DeviceInfo.DeviceHostname)
                End If
                If Not vendors.Contains(device.DeviceInfo.DeviceVendor) Then
                    vendors.Add(device.DeviceInfo.DeviceVendor)
                End If
                Dim theVersion As String = device.DeviceInfo.ClientVersion
                If String.IsNullOrEmpty(theVersion) Then theVersion = "(null)"
                If Not versions.Contains(theVersion) Then
                    versions.Add(theVersion)
                End If
                Dim lastAccessedTime As DateTime = Convert.ToDateTime(device.LastAccessedTime)
                Dim mins As Double = (DateTime.Now - lastAccessedTime).TotalMinutes
                If mins > theMinsOld Then
                    noContactdeviceNames.Add(device.DeviceInfo.DeviceName)
                    noContactIPAddresses.Add(device.DeviceInfo.DeviceIP)
                    noContactDeviceAndIp.Add(device.DeviceInfo.DeviceName + "," + device.DeviceInfo.DeviceIP)
                End If

            Next

            ' Device Total Count
            Dim count As Int32 = data.DeviceConfigurations.Count
            Dim values As String() = deviceThreshold.Split(New Char() {","c})
            Dim theOperator As String = values(0)
            Dim theThreshold As String = values(1)

            If Not TestThreshold(count, theThreshold, theOperator) Then
                success = False
                result.AppendFormat("Error: The KTA server {0} MFP device total count is {1} devices, not {2} the {3} threshold limit.<br />",
                            serverAddress, count, OperatorToText(theOperator).ToLower(), theThreshold)
            End If

            If Not String.IsNullOrEmpty(location) Then location = String.Format(";Location={0}", location)

            If Not String.IsNullOrEmpty(metricDeviceCount) Then
                resultMetric.AppendFormat("<Metric>{0}={1};Hostnames={2}{3}</Metric><br />", metricDeviceCount, count, String.Join(",", hostnames), location)
            End If

            ' Vendor Total Count
            count = vendors.Count
            values = vendorThreshold.Split(New Char() {","c})
            theOperator = values(0)
            theThreshold = values(1)

            If Not TestThreshold(count, theThreshold, theOperator) Then
                success = False
                result.AppendFormat("Error: The KTA server {0} MFP device vendor total count is {1} vendors, not {2} the {3} threshold limit.<br />",
                            serverAddress, count, OperatorToText(theOperator).ToLower(), theThreshold)
            End If

            If Not String.IsNullOrEmpty(metricVendorCount) Then
                resultMetric.AppendFormat("<Metric>{0}={1};Vendors={2}{3}</Metric><br />", metricVendorCount, count, String.Join(",", vendors), location)
            End If

            ' Verson Total Count
            count = versions.Count
            values = versionCountThreshold.Split(New Char() {","c})
            theOperator = values(0)
            theThreshold = values(1)

            If Not TestThreshold(count, theThreshold, theOperator) Then
                success = False
                result.AppendFormat("Error: The KTA server {0} MFP device client version total count is {1} versions, not {2} the {3} threshold limit.<br />",
                            serverAddress, count, OperatorToText(theOperator).ToLower(), theThreshold)
            End If

            If Not String.IsNullOrEmpty(metricVersionCount) Then
                resultMetric.AppendFormat("<Metric>{0}={1};Versions={2}{3}</Metric><br />", metricVersionCount, count, String.Join(",", versions), location)
            End If

            ' Verson Count

            If Not String.IsNullOrEmpty(versionThreshold) Then
                Dim versionvalues = versionThreshold.Split(New Char() {";"c})
                For Each threshold As String In versionvalues
                    values = threshold.Split(New Char() {","c})
                    count = versions.Count
                    Dim theVersion As String = values(0)
                    theOperator = values(1)
                    theThreshold = values(2)
                    If theVersion = "(null)" Then theVersion = ""

                    count = 0
                    For Each device In data.DeviceConfigurations
                        If device.DeviceInfo.ClientVersion Like theVersion Then count += 1
                    Next

                    If Not TestThreshold(count, theThreshold, theOperator) Then
                        success = False
                        If String.IsNullOrEmpty(theVersion) Then theVersion = "(null)"
                        result.AppendFormat("Error: The KTA server {0} MFP device client version count is {1} for client {2} version, not {3} the {4} threshold limit.<br />",
                    serverAddress, count, theVersion, OperatorToText(theOperator).ToLower(), theThreshold)
                    End If
                Next
            End If


            ' No Contact Total Count
            count = noContactdeviceNames.Count
            values = noContactThreshold.Split(New Char() {","c})
            theOperator = values(0)
            theThreshold = values(1)

            If Not TestThreshold(count, theThreshold, theOperator) Then
                success = False
                result.AppendFormat("Error: The KTA server {0} MFP device no contact total count is {1} devices, not {2} the {3} threshold limit for the last {4} minutes.<br />",
                            serverAddress, count, OperatorToText(theOperator).ToLower(), theThreshold, theMinsOld)
                result.AppendFormat("The no contact MFP devices are: {0}<br /><br />", String.Join(";", noContactDeviceAndIp))
            End If

            If Not String.IsNullOrEmpty(metricNoContact) Then
                resultMetric.AppendFormat("<Metric>{0}={1};DeviceNames={2};DeviceAddresses={3}{4}</Metric><br />",
                                          metricNoContact, count, String.Join(",", noContactdeviceNames), String.Join(",", noContactIPAddresses), location)
            End If

            If success Then
                result.AppendFormat(ReveilleSystem.Constants.Kofax_Success + "<br /><br />")
            End If

        End If
    Catch ex As Exception
        result.Append("Error: Exception occurred.<br />")
        result.AppendFormat("Exception Source:  {0}<br />", ex.Source)
        result.AppendFormat("Exception Message:  {0}<br />", ex.Message)
        result.AppendFormat("Exception Stack trace:<br />{0}<br />", ex.StackTrace)
    End Try

    Response.Write(result.ToString())

    Response.Write(resultMetric.ToString())

    If isDebug Then
        Response.Write(resultDebug.ToString())
    End If

    Response.End()
%>
    
<!--#Include File="..\FileAccess\CommonCode.inc" -->
<!--#Include File="KofaxEventLogOnCall.inc" -->


<script runat="server">

    '
    ' Tests a value against a threshold and operator
    '
    Private Function TestThreshold(value As Int32, threshold As String, operatorType As String) As Boolean
        TestThreshold = False
        If operatorType <> "Between" Then
            TestThreshold = Evaluate(value, CInt(threshold), Nothing, operatorType)
        Else
            Dim strValue1 As String = String.Empty, strValue2 As String = String.Empty

            Call GetResponseValues(threshold, strValue1, strValue2)
            TestThreshold = Evaluate(CDbl(value), CDbl(strValue1), CDbl(strValue2), operatorType)
        End If
    End Function
</script>